@@ -2,7 +2,7 @@
 // todo: Probably not in usable state yet; need to cross-check RM and clock tree.
 
 use crate::{
-    clocks::{ClocksValid, SpeedError},
+    clocks::SpeedError,
     pac::{FLASH, PWR, RCC, SYSCFG},
 };
 
@@ -52,35 +52,6 @@ impl InputSrc {
     }
 }
 
-#[derive(Clone, Copy)]
-#[repr(u8)]
-/// RCC_cfgr2
-pub enum Prediv {
-    Div1 = 0b0000,
-    Div2 = 0b0001,
-    Div3 = 0b0010,
-    Div4 = 0b0011,
-    Div5 = 0b0100,
-    Div6 = 0b0101,
-    Div7 = 0b0110,
-    Div8 = 0b0111,
-}
-
-impl Prediv {
-    pub fn value(&self) -> u8 {
-        match self {
-            Self::Div1 => 1,
-            Self::Div2 => 2,
-            Self::Div3 => 3,
-            Self::Div4 => 4,
-            Self::Div5 => 5,
-            Self::Div6 => 6,
-            Self::Div7 => 7,
-            Self::Div8 => 8,
-        }
-    }
-}
-
 #[derive(Clone, Copy)]
 #[repr(u8)]
 /// Division factor for the AHB clock. Also known as AHB Prescaler. See RCC_D1CFGR reg.
@@ -135,6 +106,17 @@ impl ApbPrescaler {
     }
 }
 
+#[derive(Clone, Copy, PartialEq)]
+#[repr(u8)]
+/// SAI clock input source. Sets RCC_D2CCIP1R register, SAIxSEL fields.
+pub enum SaiSrc {
+    Pll1Q = 0b000,
+    Pll2P = 0b001,
+    Pll3P = 0b010,
+    I2sCkin = 0b011,
+    PerClk = 0100,
+}
+
 #[derive(Clone, Copy, PartialEq)]
 #[repr(u8)]
 /// Clock divider for the HSI. See RCC_CR register, HSIDIV field.
@@ -190,7 +172,7 @@ impl VosRange {
                 185_000_001..=210_000_000 => (2, 2),
                 210_000_001..=225_000_000 => (3, 2),
                 225_000_001..=240_000_000 => (4, 2),
-                _ => panic!("Can't set higher than 240Mhz HCLK with VSO0 range."),
+                _ => panic!("Can't set higher than 240Mhz HCLK with VSO0 range. (Try changing the `vos_range` setting)."),
             },
             Self::VOS1 => match hclk {
                 0..=70_000_000 => (0, 0),
@@ -198,14 +180,14 @@ impl VosRange {
                 140_000_001..=185_000_000 => (2, 1),
                 185_000_001..=210_000_000 => (2, 2),
                 210_000_001..=225_000_000 => (3, 2),
-                _ => panic!("Can't set higher than 225Mhz HCLK with VOS1 range."),
+                _ => panic!("Can't set higher than 225Mhz HCLK with VOS1 range. (Try changing the `vos_range` setting)."),
             },
             Self::VOS2 => match hclk {
                 0..=55_000_000 => (0, 0),
                 55_000_001..=110_000_000 => (1, 1),
                 110_000_001..=165_000_000 => (2, 1),
                 165_000_001..=225_000_000 => (3, 2),
-                _ => panic!("Can't set higher than 225Mhz HCLK with VSO2 range."),
+                _ => panic!("Can't set higher than 225Mhz HCLK with VSO2 range. (Try changing the `vos_range` setting)."),
             },
             Self::VOS3 => match hclk {
                 0..=45_000_000 => (0, 0),
@@ -213,7 +195,7 @@ impl VosRange {
                 90_000_001..=135_000_000 => (2, 1),
                 135_000_001..=180_000_000 => (3, 2),
                 180_000_001..=225_000_000 => (4, 2),
-                _ => panic!("Can't set higher than 225Mhz HCLK with VSO3 range."),
+                _ => panic!("Can't set higher than 225Mhz HCLK with VSO3 range. (Try changing the `vos_range` setting)."),
             },
         }
     }
@@ -254,6 +236,10 @@ pub struct Clocks {
     pub divq3: u8,
     /// PLL3 DIVR division factor
     pub divr3: u8,
+    /// Enable PLL2
+    pub pll2_enable: bool,
+    /// Enable PLL3
+    pub pll3_enable: bool,
     pub d1_core_prescaler: HclkPrescaler,
     pub d1_prescaler: ApbPrescaler,
     pub hclk_prescaler: HclkPrescaler,
@@ -266,6 +252,10 @@ pub struct Clocks {
     pub security_system: bool,
     pub hsi48_on: bool,
     pub vos_range: VosRange,
+    /// SAI1 and DFSDM1 kernel Aclk clock source selection
+    pub sai1_src: SaiSrc,
+    /// SAI2 and SAI3 kernel clock source selection
+    pub sai23_src: SaiSrc,
 }
 
 impl Clocks {
@@ -283,8 +273,8 @@ impl Clocks {
         pwr: &mut PWR,
         syscfg: &mut SYSCFG,
     ) -> Result<(), SpeedError> {
-        if let ClocksValid::NotValid = self.validate_speeds() {
-            return Err(SpeedError {});
+        if let Err(e) = self.validate_speeds() {
+            return Err(e);
         }
 
         // Enable and reset System Configuration Controller, ie for interrupts.
@@ -305,7 +295,7 @@ impl Clocks {
         match self.vos_range {
             #[cfg(not(feature = "h7b3"))]
             VosRange::VOS0 => {
-                // VOS0 activation/deactivation sequence
+                // VOS0 activation/deactivation sequence: H743 HRM, section 6.6.2:
                 // The system maximum frequency can be reached by boosting the voltage scaling level to
                 // VOS0. This is done through the ODEN bit in the SYSCFG_PWRCR register.
                 // The sequence to activate the VOS0 is the following:
@@ -325,7 +315,7 @@ impl Clocks {
                     if #[cfg(any(feature = "h747cm4", feature = "h747cm7"))] {
                         syscfg.pwrcr.modify(|_, w| w.oden().set_bit());
                     } else {
-                        syscfg.pwrcr.write(|w| unsafe { w.oden().bits(1) });
+                        syscfg.pwrcr.modify(|_, w| unsafe { w.oden().bits(1) });
                     }
                 }
 
@@ -348,12 +338,9 @@ impl Clocks {
 
         // Adjust flash wait states according to the HCLK frequency.
         // We need to do this before enabling PLL, or it won't enable.
-        let sysclk = self.sysclk();
-
-        let hclk = sysclk / self.hclk_prescaler.value() as u32;
         // H742 RM, Table 17.
         // todo: What should this be?
-        let wait_states = self.vos_range.wait_states(hclk);
+        let wait_states = self.vos_range.wait_states(self.hclk());
         flash.acr.modify(|_, w| unsafe {
             w.latency().bits(wait_states.0);
             w.wrhighfreq().bits(wait_states.1)
@@ -431,6 +418,8 @@ impl Clocks {
             w.usbsel().bits(0b11)
         });
 
+        // todo: Allow configuring the PLL in fractinoal mode.
+
         if let InputSrc::Pll1(pll_src) = self.input_src {
             // Turn off the PLL: Required for modifying some of the settings below.
             rcc.cr.modify(|_, w| w.pll1on().clear_bit());
@@ -446,7 +435,7 @@ impl Clocks {
 
             // Set and reset by software to select the proper reference frequency range used for PLL1.
             // This bit must be written before enabling the PLL1.
-            let pll1_rng_val = match self.pll_input_speed(pll_src) {
+            let pll1_rng_val = match self.pll_input_speed(pll_src, 1) {
                 1_000_000..=2_000_000 => 0b00,
                 2_000_001..=4_000_000 => 0b01,
                 4_000_001..=8_000_000 => 0b10,
@@ -454,41 +443,129 @@ impl Clocks {
                 _ => panic!("PLL1 input source must be between 1Mhz and 16Mhz."),
             };
 
-            // todo: Use PLLCFGR to enable the various PLL1, 2, 3 R, Q, and P outputs as required
-            rcc.pllcfgr.modify(|_, w| w.pll1rge().bits(pll1_rng_val));
+            // todo DRY!
+            let pll2_rng_val = match self.pll_input_speed(pll_src, 2) {
+                1_000_000..=2_000_000 => 0b00,
+                2_000_001..=4_000_000 => 0b01,
+                4_000_001..=8_000_000 => 0b10,
+                8_000_001..=16_000_000 => 0b11,
+                _ => panic!("PLL2 input source must be between 1Mhz and 16Mhz."),
+            };
+
+            // todo DRY!
+            let pll3_rng_val = match self.pll_input_speed(pll_src, 3) {
+                1_000_000..=2_000_000 => 0b00,
+                2_000_001..=4_000_000 => 0b01,
+                4_000_001..=8_000_000 => 0b10,
+                8_000_001..=16_000_000 => 0b11,
+                _ => panic!("PLL3 input source must be between 1Mhz and 16Mhz."),
+            };
+
+            // todo: Don't enable all these pqr etc by default!
+            // todo don't enable pll2 and 3 by default!!
+            // todo: This has an impact on power consumption.
+
+            // todo: MOre DRY
+            // H743 RM:
+            // 0: Wide VCO range: 192 to 836 MHz (default after reset)
+            // 1: Medium VCO range: 150 to 420 MHz
+            let pll1_vco = match self.pll_input_speed(pll_src, 1) {
+                0..=2_000_000 => 0,
+                _ => 1,
+            };
+            let pll2_vco = match self.pll_input_speed(pll_src, 2) {
+                0..=2_000_000 => 0,
+                _ => 1,
+            };
+            let pll3_vco = match self.pll_input_speed(pll_src, 3) {
+                0..=2_000_000 => 0,
+                _ => 1,
+            };
+
+            // The user application can then configure the proper VCO: if the frequency of the reference
+            // clock is lower or equal to 2 MHz, then VCOL must be selected, otherwise VCOH must be
+            // chosen. To reduce the power consumption, it is recommended to configure the VCO output
+            // to the lowest frequency.
+
+            // The frequency of the reference clock provided to the PLLs (refx_ck) must range from 1 to
+            // 16 MHz. The user application has to program properly the DIVMx dividers of the RCC PLLs
+            // Clock Source Selection Register (RCC_PLLCKSELR) in order to match this condition. In
+            // addition, the PLLxRGE of the RCC PLLs Configuration Register (RCC_PLLCFGR) field
+            // must be set according to the reference input frequency to guarantee an optimal
+            // performance of the PLL.
+
+            // struct PllEnCfg {
+            //     p_en: bool,
+            //     q_en: bool,
+            //     r_en: bool
+            //
+
+            rcc.pllcfgr.modify(|_, w| {
+                w.pll1rge().bits(pll1_rng_val);
+                w.pll1vcosel().bit(pll1_vco != 0);
+                w.divp1en().set_bit()
+                // w.divr1en().set_bit();
+                // w.divq1en().set_bit();
+            });
 
             rcc.pll1divr.modify(|_, w| unsafe {
-                w.divn1().bits(self.divn1);
-                w.divp1().bits(self.divp1);
-                w.divq1().bits(self.divq1);
-                w.divr1().bits(self.divr1)
-            });
-
-            rcc.pll2divr.modify(|_, w| unsafe {
-                w.divn2().bits(self.divn2);
-                w.divp2().bits(self.divp2);
-                w.divq2().bits(self.divq2);
-                w.divr2().bits(self.divr2)
-            });
-
-            rcc.pll3divr.modify(|_, w| unsafe {
-                w.divn3().bits(self.divn3);
-                w.divp3().bits(self.divp3);
-                w.divq3().bits(self.divq3);
-                w.divr3().bits(self.divr3)
+                w.divn1().bits(self.divn1 - 1);
+                w.divp1().bits(self.divp1 - 1);
+                w.divq1().bits(self.divq1 - 1);
+                w.divr1().bits(self.divr1 - 1)
             });
 
             // Now turn PLL back on, once we're configured things that can only be set with it off.
-            // todo: Enable sai1 and 2 with separate settings, or lump in with mail PLL
-            // like this?
             rcc.cr.modify(|_, w| w.pll1on().set_bit());
             while rcc.cr.read().pll1rdy().bit_is_clear() {}
 
-            // todo: Enabling other PLLs?
+            if self.pll2_enable {
+                rcc.pllcfgr.modify(|_, w| {
+                    w.pll2rge().bits(pll2_rng_val);
+                    w.pll2vcosel().bit(pll2_vco != 0);
+                    w.divp2en().set_bit();
+                    w.divr2en().set_bit();
+                    w.divq2en().set_bit()
+                });
 
-            // todo: SAI clock?
+                rcc.pll2divr.modify(|_, w| unsafe {
+                    w.divn2().bits(self.divn2 - 1);
+                    w.divp2().bits(self.divp2 - 1);
+                    w.divq2().bits(self.divq2 - 1);
+                    w.divr2().bits(self.divr2 - 1)
+                });
+
+                rcc.cr.modify(|_, w| w.pll2on().set_bit());
+                while rcc.cr.read().pll2rdy().bit_is_clear() {}
+            }
+
+            if self.pll3_enable {
+                rcc.pllcfgr.modify(|_, w| {
+                    w.pll3rge().bits(pll3_rng_val);
+                    w.pll3vcosel().bit(pll3_vco != 0);
+                    w.divp3en().set_bit();
+                    w.divr3en().set_bit();
+                    w.divq3en().set_bit()
+                });
+
+                rcc.pll3divr.modify(|_, w| unsafe {
+                    w.divn3().bits(self.divn3 - 1);
+                    w.divp3().bits(self.divp3 - 1);
+                    w.divq3().bits(self.divq3 - 1);
+                    w.divr3().bits(self.divr3 - 1)
+                });
+
+                rcc.cr.modify(|_, w| w.pll3on().set_bit());
+                while rcc.cr.read().pll3rdy().bit_is_clear() {}
+            }
         }
 
+        #[cfg(not(feature = "h7b3"))]
+        rcc.d2ccip1r.modify(|_, w| unsafe {
+            w.sai1sel().bits(self.sai1_src as u8);
+            w.sai23sel().bits(self.sai23_src as u8)
+        });
+
         rcc.cr.modify(|_, w| w.hsecsson().bit(self.security_system));
 
         if self.hsi48_on {
@@ -561,15 +638,33 @@ impl Clocks {
         }
     }
 
-    /// Calculate the input speed to the PLL
-    fn pll_input_speed(&self, pll_src: PllSrc) -> u32 {
+    /// Calculate the input speed to the PLL. This must be between 1 and 16 Mhz. Called `refx_ck`
+    /// in the RM.
+    pub fn pll_input_speed(&self, pll_src: PllSrc, pll_num: u8) -> u32 {
         let input_freq = match pll_src {
             PllSrc::Csi => 4_000_000,
             PllSrc::Hsi(div) => 64_000_000 / (div.value() as u32),
             PllSrc::Hse(freq) => freq,
             PllSrc::None => 0,
         };
-        input_freq / self.divm1 as u32
+
+        match pll_num {
+            1 => input_freq / (self.divm1 as u32),
+            2 => input_freq / (self.divm2 as u32),
+            3 => input_freq / (self.divm3 as u32),
+            _ => panic!("Pll num must be between 1 and 3."),
+        }
+    }
+
+    /// Calculate VCO output frequency: = Fref1_ck x DIVN1
+    pub fn vco_output_freq(&self, pll_src: PllSrc, pll_num: u8) -> u32 {
+        let input_speed = self.pll_input_speed(pll_src, pll_num);
+        match pll_num {
+            1 => input_speed * self.divn1 as u32,
+            2 => input_speed * self.divn2 as u32,
+            3 => input_speed * self.divn3 as u32,
+            _ => panic!("Pll num must be between 1 and 3."),
+        }
     }
 
     /// Check if the PLL is enabled. This is useful if checking wheather to re-enable the PLL
@@ -587,7 +682,8 @@ impl Clocks {
     pub fn sysclk(&self) -> u32 {
         match self.input_src {
             InputSrc::Pll1(pll_src) => {
-                self.pll_input_speed(pll_src) * self.divn1 as u32 / self.divp1 as u32
+                // divm1 is included in `pll_input_speed`.
+                self.pll_input_speed(pll_src, 1) * self.divn1 as u32 / self.divp1 as u32
             }
             InputSrc::Csi => 4_000_000,
             InputSrc::Hsi(div) => 64_000_000 / (div.value() as u32),
@@ -595,13 +691,17 @@ impl Clocks {
         }
     }
 
+    pub fn d1cpreclk(&self) -> u32 {
+        self.sysclk() / self.d1_core_prescaler.value() as u32
+    }
+
     pub fn hclk(&self) -> u32 {
         self.sysclk() / self.d1_core_prescaler.value() as u32 / self.hclk_prescaler.value() as u32
     }
 
     pub fn systick(&self) -> u32 {
         // todo: There's an optional /8 divider we're not taking into account here.
-        self.hclk()
+        self.d1cpreclk()
     }
 
     pub fn usb(&self) -> u32 {
@@ -634,9 +734,7 @@ impl Clocks {
         }
     }
 
-    pub fn validate_speeds(&self) -> ClocksValid {
-        let mut result = ClocksValid::Valid;
-
+    pub fn validate_speeds(&self) -> Result<(), SpeedError> {
         // todo: This depends on variant
         // let max_clock = 550_000_000;
         // #[cfg(feature = "h743")]
@@ -655,49 +753,70 @@ impl Clocks {
             || self.divm2 > 63
             || self.divm3 > 63
             || self.divn1 > 512
+            || self.divn2 > 512
+            || self.divn3 > 512
             || self.divp1 > 128
+            || self.divp2 > 128
+            || self.divp3 > 128
             || self.divp1 < 2
+            || self.divp2 < 2
+            || self.divp2 < 2
         {
-            return ClocksValid::NotValid;
+            return Err(SpeedError::new("A PLL divider is out of limits"));
         }
 
         if let InputSrc::Pll1(pll_src) = self.input_src {
-            let pll_input_speed = self.pll_input_speed(pll_src);
+            let pll_input_speed = self.pll_input_speed(pll_src, 1);
             if pll_input_speed < 1_000_000 || pll_input_speed > 16_000_000 {
-                return ClocksValid::NotValid;
+                return Err(SpeedError::new("Invalid PLL input speed"));
+            }
+            // VCO0: Wide VCO range: 192 to 836 MHz (default after reset) (VCOH)
+            // Note: The RM appears out of date: Revision "V" allgedly supports 960_000_000
+            // VCO speed, to allow a max core speed of 480Mhz.
+            let vco_speed = self.vco_output_freq(pll_src, 1);
+            if pll_input_speed <= 2_000_000 && (vco_speed < 192_000_000 || vco_speed > 960_000_000)
+            {
+                return Err(SpeedError::new("Invalid wide VCO speed"));
+            }
+            // 1: Medium VCO range: 150 to 420 MHz. (VCOL)
+            // Note: You may get power savings
+            if pll_input_speed > 2_000_000 && (vco_speed < 150_000_000 || vco_speed > 420_000_000) {
+                return Err(SpeedError::new("Invalid medium VCO speed"));
             }
         }
 
+        // todo: Validate PLL2 and PLL3 speeds.
+
         // todo: More work on this, including feature gates
 
         // todo: QC these limits
         // todo: Note that this involves repeatedly calculating sysclk.
         // todo. We could work around thsi by calcing it once here.
         if self.sysclk() > max_sysclk {
-            result = ClocksValid::NotValid;
+            return Err(SpeedError::new("Sysclock out of limits"));
         }
 
         if self.hclk() > max_hclk {
-            result = ClocksValid::NotValid;
+            return Err(SpeedError::new("Hclk out of limits"));
         }
 
         if self.apb1() > max_apb {
-            result = ClocksValid::NotValid;
+            return Err(SpeedError::new("APB1 out of limits"));
         }
 
         if self.apb2() > max_apb {
-            result = ClocksValid::NotValid;
+            return Err(SpeedError::new("APB2 out of limits"));
         }
 
         // todo: Apb3/4?
 
-        result
+        Ok(())
     }
 }
 
 impl Default for Clocks {
     /// This default configures clocks with the HSI, and a 240Mhz sysclock speed.
-    /// Peripheral clocks are all set to 112Mhz, with timer clocks at 224Mhz.
+    /// Peripheral clocks are all set to 112.5Mhz, with timer clocks at 225Mhz.
     /// HSE output is not bypassed.
     /// todo: Not 248Mhz due to an issue with wait states on Table 17 not showing higher than 225Mhz
     /// // todo HCLK unless in VOS0.
@@ -705,27 +824,29 @@ impl Default for Clocks {
         // todo: Feature-gate based on variant. Ie the 5xxMhz ones, the 240Mhz ones, and dual cores.
         Self {
             input_src: InputSrc::Pll1(PllSrc::Hsi(HsiDiv::Div1)),
-            divm1: 8,
+            divm1: 32,
             // #[cfg(feature = "h743")]
             // divn1: 240, // todo: For 480Mhz with right vco set.
             // #[cfg(feature = "h743")]
-            divn1: 56,
+            divn1: 225,
             divp1: 2,
             divq1: 3, // Allows <150Mhz SAI clock, if it's configureud for PLL1Q.
             divr1: 2,
-            divm2: 1,
-            divn2: 129,
+            divm2: 32,
+            divn2: 225,
             divp2: 2,
             divq2: 2,
             divr2: 2,
-            divm3: 1,
-            divn3: 129,
+            divm3: 32,
+            divn3: 225,
             divp3: 2,
             divq3: 2,
             divr3: 2,
+            pll2_enable: false,
+            pll3_enable: false,
             d1_core_prescaler: HclkPrescaler::Div1,
             d1_prescaler: ApbPrescaler::Div1,
-            hclk_prescaler: HclkPrescaler::Div1,
+            hclk_prescaler: HclkPrescaler::Div2,
             d2_prescaler1: ApbPrescaler::Div2,
             d2_prescaler2: ApbPrescaler::Div2,
             d3_prescaler: ApbPrescaler::Div2,
@@ -733,6 +854,8 @@ impl Default for Clocks {
             security_system: false,
             hsi48_on: false,
             vos_range: VosRange::VOS1,
+            sai1_src: SaiSrc::Pll1Q,
+            sai23_src: SaiSrc::Pll1Q,
         }
     }
 }
